self.__precacheManifest = (self.__precacheManifest || []).concat([
  {
    "revision": "072f0c589ce4c78f4d5004bc2f1d4b1f",
    "url": "/static/index.html"
  },
  {
    "revision": "881956a8b6f1af3d12f0",
    "url": "/static/static/css/main.fc37acba.chunk.css"
  },
  {
    "revision": "072d2e616e83eff14461",
    "url": "/static/static/js/2.3cdb765f.chunk.js"
  },
  {
    "revision": "881956a8b6f1af3d12f0",
    "url": "/static/static/js/main.30eaf701.chunk.js"
  },
  {
    "revision": "517784e95ca40345faae",
    "url": "/static/static/js/runtime~main.e6c9c1d2.js"
  }
]);